// Copyright © Microsoft Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.opengroup.osdu.register.provider.azure.subscriber;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.jupiter.MockitoExtension;
import org.opengroup.osdu.core.common.logging.JaxRsDpsLog;
import org.opengroup.osdu.register.provider.interfaces.subscriber.ISubscriptionRepository;
import org.opengroup.osdu.register.subscriber.model.HmacSecret;
import org.opengroup.osdu.register.subscriber.model.Subscription;

import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
public class TestSubscriptionAzureTest {
    @Mock
    private ISubscriptionRepository subscriptionRepository;
    @Mock
    private JaxRsDpsLog logger;
    @InjectMocks
    private TestSubscriptionAzure sut;

    @Test
    public void createTestAcknowledgementSubscription() throws Exception {
        when(subscriptionRepository.create(getSubscription())).thenReturn(getSubscription());

        sut.performTestAcknowledgement();

        verify(logger, times(1)).debug("Successfully created Acknowledgement Subscription");
        verify(subscriptionRepository, times(1)).create(getSubscription());
    }

    @Test
    public void shouldThrowExceptionWhenSubscriptionCreationFailed() throws Exception {
        when(subscriptionRepository.create(getSubscription())).thenThrow(new Exception());

        Exception exception = assertThrows(Exception.class, () -> {
            sut.performTestAcknowledgement();
        });

        assertNotNull(exception);
        verify(logger, times(0)).debug("Successfully created Acknowledgement Subscription");
        verify(subscriptionRepository, times(1)).create(getSubscription());
    }

    private Subscription getSubscription() {
        Subscription subscription = new Subscription();
        subscription.setName("AcknowledgementTestSubscription");
        subscription.setDescription("Subscription for Ack of test-endpoint");
        subscription.setTopic("recordstopic");
        subscription.setPushEndpoint("http://TestAck/api/register/v1/test/challenge/1");
        subscription.setNotificationId("testingAcknowledgement");
        subscription.setId("AckSubscription");
        HmacSecret secret = new HmacSecret();
        secret.setValue("TestAcknowledgementSubscription");
        subscription.setSecret(secret);
        return subscription;
    }
}
